temporal_sdk_api_workflow_check behaviour (temporal_sdk v0.1.17)
View SourceWorkflow determinism check behaviour module.
The SDK provides two built-in implementations of the temporal_sdk_api_workflow_check behaviour:
temporal_sdk_api_workflow_check_temporal (default) and temporal_sdk_api_workflow_check_strict.
In the temporal_sdk_api_workflow_check_temporal implementation, actual and replayed
awaitables pass the determinism check if their kind matches.
In the temporal_sdk_api_workflow_check_strict implementation, actual and replayed awaitables
pass the determinism check only if their full specifications match - their kind, type, and
id/name must all be identical.
See also: GitHub SDK Samples - Determinism Check.
Summary
Callbacks
Determinism check of actual versus replayed awaitables, commands and workflow history events.
Callbacks
-callback is_deterministic(ActualAwaitable :: temporal_sdk_workflow:awaitable_temporal_index(), ReplayedAwaitable :: temporal_sdk_workflow:awaitable_temporal_index(), ActualCommand :: temporal_sdk_proto_service_workflow_binaries:'temporal.api.command.v1.Command'(), ReplayedHistoryEvent :: temporal_sdk_workflow:history_event()) -> boolean().
Determinism check of actual versus replayed awaitables, commands and workflow history events.